@@ -5,12 +5,10 @@ import (
 	"fmt"
 	"log"
 	"os"
-	"os/signal"
 	"path/filepath"
 	"sort"
 	"strings"
 	"sync"
-	"syscall"
 	"text/tabwriter"
 
 	"github.com/variantdev/vals"
@@ -24,6 +22,8 @@ import (
 	"github.com/helmfile/helmfile/pkg/state"
 )
 
+var CleanWaitGroup sync.WaitGroup
+
 // App is the main application object.
 type App struct {
 	OverrideKubeContext string
@@ -93,6 +93,14 @@ func Init(app *App) *App {
 	return app
 }
 
+func (a *App) Init(c InitConfigProvider) error {
+	runner := &helmexec.ShellRunner{
+		Logger: a.Logger,
+	}
+	helmfileInit := NewHelmfileInit(a.OverrideHelmBinary, c, a.Logger, runner)
+	return helmfileInit.Initialize()
+}
+
 func (a *App) Deps(c DepsConfigProvider) error {
 	return a.ForEachState(func(run *Run) (_ bool, errs []error) {
 		prepErr := run.withPreparedCharts("deps", state.ChartPrepareOptions{
@@ -159,11 +167,12 @@ func (a *App) Diff(c DiffConfigProvider) error {
 		includeCRDs := !c.SkipCRDs()
 
 		prepErr := run.withPreparedCharts("diff", state.ChartPrepareOptions{
-			SkipRepos:   c.SkipDeps(),
-			SkipDeps:    c.SkipDeps(),
-			IncludeCRDs: &includeCRDs,
-			Validate:    c.Validate(),
-			Concurrency: c.Concurrency(),
+			SkipRepos:              c.SkipDeps(),
+			SkipDeps:               c.SkipDeps(),
+			IncludeCRDs:            &includeCRDs,
+			Validate:               c.Validate(),
+			Concurrency:            c.Concurrency(),
+			IncludeTransitiveNeeds: c.IncludeNeeds(),
 		}, func() {
 			msg, matched, affected, errs = a.diff(run, c)
 		})
@@ -225,13 +234,14 @@ func (a *App) Template(c TemplateConfigProvider) error {
 		// `helm template` in helm v2 does not support local chart.
 		// So, we set forceDownload=true for helm v2 only
 		prepErr := run.withPreparedCharts("template", state.ChartPrepareOptions{
-			ForceDownload: !run.helm.IsHelm3(),
-			SkipRepos:     c.SkipDeps(),
-			SkipDeps:      c.SkipDeps(),
-			IncludeCRDs:   &includeCRDs,
-			SkipCleanup:   c.SkipCleanup(),
-			Validate:      c.Validate(),
-			Concurrency:   c.Concurrency(),
+			ForceDownload:          !run.helm.IsHelm3(),
+			SkipRepos:              c.SkipDeps(),
+			SkipDeps:               c.SkipDeps(),
+			IncludeCRDs:            &includeCRDs,
+			SkipCleanup:            c.SkipCleanup(),
+			Validate:               c.Validate(),
+			Concurrency:            c.Concurrency(),
+			IncludeTransitiveNeeds: c.IncludeNeeds(),
 		}, func() {
 			ok, errs = a.template(run, c)
 		})
@@ -300,11 +310,12 @@ func (a *App) Lint(c LintConfigProvider) error {
 
 		// `helm lint` on helm v2 and v3 does not support remote charts, that we need to set `forceDownload=true` here
 		prepErr := run.withPreparedCharts("lint", state.ChartPrepareOptions{
-			ForceDownload: true,
-			SkipRepos:     c.SkipDeps(),
-			SkipDeps:      c.SkipDeps(),
-			SkipCleanup:   c.SkipCleanup(),
-			Concurrency:   c.Concurrency(),
+			ForceDownload:          true,
+			SkipRepos:              c.SkipDeps(),
+			SkipDeps:               c.SkipDeps(),
+			SkipCleanup:            c.SkipCleanup(),
+			Concurrency:            c.Concurrency(),
+			IncludeTransitiveNeeds: c.IncludeNeeds(),
 		}, func() {
 			ok, lintErrs, errs = a.lint(run, c)
 		})
@@ -334,11 +345,12 @@ func (a *App) Lint(c LintConfigProvider) error {
 func (a *App) Fetch(c FetchConfigProvider) error {
 	return a.ForEachState(func(run *Run) (ok bool, errs []error) {
 		prepErr := run.withPreparedCharts("pull", state.ChartPrepareOptions{
-			ForceDownload: true,
-			SkipRepos:     c.SkipDeps(),
-			SkipDeps:      c.SkipDeps(),
-			OutputDir:     c.OutputDir(),
-			Concurrency:   c.Concurrency(),
+			ForceDownload:     true,
+			SkipRepos:         c.SkipDeps(),
+			SkipDeps:          c.SkipDeps(),
+			OutputDir:         c.OutputDir(),
+			OutputDirTemplate: c.OutputDirTemplate(),
+			Concurrency:       c.Concurrency(),
 		}, func() {
 		})
 
@@ -360,7 +372,7 @@ func (a *App) Sync(c SyncConfigProvider) error {
 			Wait:                   c.Wait(),
 			WaitForJobs:            c.WaitForJobs(),
 			IncludeCRDs:            &includeCRDs,
-			IncludeTransitiveNeeds: c.IncludeTransitiveNeeds(),
+			IncludeTransitiveNeeds: c.IncludeNeeds(),
 			Validate:               c.Validate(),
 			Concurrency:            c.Concurrency(),
 		}, func() {
@@ -388,14 +400,15 @@ func (a *App) Apply(c ApplyConfigProvider) error {
 		includeCRDs := !c.SkipCRDs()
 
 		prepErr := run.withPreparedCharts("apply", state.ChartPrepareOptions{
-			SkipRepos:   c.SkipDeps(),
-			SkipDeps:    c.SkipDeps(),
-			Wait:        c.Wait(),
-			WaitForJobs: c.WaitForJobs(),
-			IncludeCRDs: &includeCRDs,
-			SkipCleanup: c.RetainValuesFiles() || c.SkipCleanup(),
-			Validate:    c.Validate(),
-			Concurrency: c.Concurrency(),
+			SkipRepos:              c.SkipDeps(),
+			SkipDeps:               c.SkipDeps(),
+			Wait:                   c.Wait(),
+			WaitForJobs:            c.WaitForJobs(),
+			IncludeCRDs:            &includeCRDs,
+			SkipCleanup:            c.RetainValuesFiles() || c.SkipCleanup(),
+			Validate:               c.Validate(),
+			Concurrency:            c.Concurrency(),
+			IncludeTransitiveNeeds: c.IncludeNeeds(),
 		}, func() {
 			matched, updated, es := a.apply(run, c)
 
@@ -539,7 +552,12 @@ func (a *App) PrintState(c StateConfigProvider) error {
 				return
 			}
 
-			fmt.Printf("---\n#  Source: %s\n\n%+v", run.state.FilePath, stateYaml)
+			sourceFile, err := run.state.FullFilePath()
+			if err != nil {
+				errs = []error{err}
+				return
+			}
+			fmt.Printf("---\n#  Source: %s\n\n%+v", sourceFile, stateYaml)
 
 			errs = []error{}
 		})
@@ -776,7 +794,7 @@ func (a *App) getHelm(st *state.HelmState) helmexec.Interface {
 func (a *App) visitStates(fileOrDir string, defOpts LoadOpts, converge func(*state.HelmState) (bool, []error)) error {
 	noMatchInHelmfiles := true
 
-	err := a.visitStateFiles(fileOrDir, defOpts, func(f, d string) error {
+	err := a.visitStateFiles(fileOrDir, defOpts, func(f, d string) (retErr error) {
 		opts := defOpts.DeepCopy()
 
 		if opts.CalleePath == "" {
@@ -785,22 +803,6 @@ func (a *App) visitStates(fileOrDir string, defOpts LoadOpts, converge func(*sta
 
 		st, err := a.loadDesiredStateFromYaml(f, opts)
 
-		sigs := make(chan os.Signal, 1)
-		signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM)
-		go func() {
-			sig := <-sigs
-
-			errs := []error{fmt.Errorf("received [%s] to shutdown ", sig)}
-			_ = context{app: a, st: st, retainValues: defOpts.RetainValuesFiles}.clean(errs)
-			// See http://tldp.org/LDP/abs/html/exitcodes.html
-			switch sig {
-			case syscall.SIGINT:
-				os.Exit(130)
-			case syscall.SIGTERM:
-				os.Exit(143)
-			}
-		}()
-
 		ctx := context{app: a, st: st, retainValues: defOpts.RetainValuesFiles}
 
 		if err != nil {
@@ -864,7 +866,32 @@ func (a *App) visitStates(fileOrDir string, defOpts LoadOpts, converge func(*sta
 			return appError(fmt.Sprintf("failed executing release templates in \"%s\"", f), tmplErr)
 		}
 
-		processed, errs := converge(templated)
+		var (
+			processed bool
+			errs      []error
+		)
+
+		// Ensure every temporary files and directories generated while running
+		// the converge function is clean up before exiting this function in all the three cases below:
+		// - This function returned nil
+		// - This function returned an err
+		// - Helmfile received SIGINT or SIGTERM while running this function
+		// For the last case you also need a signal handler in main.go.
+		// Ideally though, this CleanWaitGroup should gone and be replaced by a context cancellation propagation.
+		// See https://github.com/helmfile/helmfile/pull/418 for more details.
+		CleanWaitGroup.Add(1)
+		defer func() {
+			defer CleanWaitGroup.Done()
+			cleanErr := context{app: a, st: templated, retainValues: defOpts.RetainValuesFiles}.clean(errs)
+			if retErr == nil {
+				retErr = cleanErr
+			} else if cleanErr != nil {
+				a.Logger.Debugf("Failed to clean up temporary files generated while processing %q: %v", templated.FilePath, cleanErr)
+			}
+		}()
+
+		processed, errs = converge(templated)
+
 		noMatchInHelmfiles = noMatchInHelmfiles && !processed
 
 		if opts.Reverse {
@@ -874,7 +901,7 @@ func (a *App) visitStates(fileOrDir string, defOpts LoadOpts, converge func(*sta
 			}
 		}
 
-		return context{app: a, st: templated, retainValues: defOpts.RetainValuesFiles}.clean(errs)
+		return nil
 	})
 
 	if err != nil {
@@ -951,13 +978,17 @@ func withDAG(templated *state.HelmState, helm helmexec.Interface, logger *zap.Su
 		return false, []error{err}
 	}
 
-	return withBatches(templated, batches, helm, logger, converge)
+	return withBatches(opts.Purpose, templated, batches, helm, logger, converge)
 }
 
-func withBatches(templated *state.HelmState, batches [][]state.Release, helm helmexec.Interface, logger *zap.SugaredLogger, converge func(*state.HelmState, helmexec.Interface) (bool, []error)) (bool, []error) {
+func withBatches(purpose string, templated *state.HelmState, batches [][]state.Release, helm helmexec.Interface, logger *zap.SugaredLogger, converge func(*state.HelmState, helmexec.Interface) (bool, []error)) (bool, []error) {
 	numBatches := len(batches)
 
-	logger.Debugf("processing %d groups of releases in this order:\n%s", numBatches, printBatches(batches))
+	if purpose == "" {
+		purpose = "processing"
+	}
+
+	logger.Debugf("%s %d groups of releases in this order:\n%s", purpose, numBatches, printBatches(batches))
 
 	any := false
 
@@ -974,7 +1005,7 @@ func withBatches(templated *state.HelmState, batches [][]state.Release, helm hel
 			releaseIds = append(releaseIds, state.ReleaseToID(&release))
 		}
 
-		logger.Debugf("processing releases in group %d/%d: %s", i+1, numBatches, strings.Join(releaseIds, ", "))
+		logger.Debugf("%s releases in group %d/%d: %s", purpose, i+1, numBatches, strings.Join(releaseIds, ", "))
 
 		batchSt := *templated
 		batchSt.Releases = targets
@@ -1351,32 +1382,33 @@ Do you really want to apply?
 	st.Releases = selectedAndNeededReleases
 
 	if !interactive || interactive && r.askForConfirmation(confMsg) {
+		if _, preapplyErrors := withDAG(st, helm, a.Logger, state.PlanOptions{Purpose: "invoking preapply hooks for", Reverse: true, SelectedReleases: toApplyWithNeeds, SkipNeeds: true}, a.WrapWithoutSelector(func(subst *state.HelmState, helm helmexec.Interface) []error {
+			for _, r := range subst.Releases {
+				release := r
+				if _, err := st.TriggerPreapplyEvent(&release, "apply"); err != nil {
+					return []error{err}
+				}
+			}
+
+			return nil
+		})); len(preapplyErrors) > 0 {
+			return true, false, preapplyErrors
+		}
+
 		r.helm.SetExtraArgs(argparser.GetArgs(c.Args(), r.state)...)
 
 		// We deleted releases by traversing the DAG in reverse order
 		if len(releasesToBeDeleted) > 0 {
 			_, deletionErrs := withDAG(st, helm, a.Logger, state.PlanOptions{Reverse: true, SelectedReleases: toDelete, SkipNeeds: true}, a.WrapWithoutSelector(func(subst *state.HelmState, helm helmexec.Interface) []error {
-				var (
-					rs             []state.ReleaseSpec
-					preapplyErrors []error
-				)
+				var rs []state.ReleaseSpec
 
 				for _, r := range subst.Releases {
 					release := r
 					if r2, ok := releasesToBeDeleted[state.ReleaseToID(&release)]; ok {
-						if _, err := st.TriggerPreapplyEvent(&r2, "apply"); err != nil {
-							preapplyErrors = append(applyErrs, err)
-							continue
-						}
-
 						rs = append(rs, r2)
 					}
 				}
 
-				if len(preapplyErrors) > 0 {
-					return preapplyErrors
-				}
-
 				subst.Releases = rs
 
 				return subst.DeleteReleasesForSync(&affectedReleases, helm, c.Concurrency())
@@ -1390,27 +1422,15 @@ Do you really want to apply?
 		// We upgrade releases by traversing the DAG
 		if len(releasesToBeUpdated) > 0 {
 			_, updateErrs := withDAG(st, helm, a.Logger, state.PlanOptions{SelectedReleases: toUpdate, Reverse: false, SkipNeeds: true, IncludeTransitiveNeeds: c.IncludeTransitiveNeeds()}, a.WrapWithoutSelector(func(subst *state.HelmState, helm helmexec.Interface) []error {
-				var (
-					rs             []state.ReleaseSpec
-					preapplyErrors []error
-				)
+				var rs []state.ReleaseSpec
 
 				for _, r := range subst.Releases {
 					release := r
 					if r2, ok := releasesToBeUpdated[state.ReleaseToID(&release)]; ok {
-						if _, err := st.TriggerPreapplyEvent(&r2, "apply"); err != nil {
-							preapplyErrors = append(applyErrs, err)
-							continue
-						}
-
 						rs = append(rs, r2)
 					}
 				}
 
-				if len(preapplyErrors) > 0 {
-					return preapplyErrors
-				}
-
 				subst.Releases = rs
 
 				syncOpts := &state.SyncOpts{
