@@ -1,6 +1,10 @@
 package app
 
-import "github.com/helmfile/helmfile/pkg/helmexec"
+import (
+	"helm.sh/helm/v3/pkg/chart"
+
+	"github.com/helmfile/helmfile/pkg/helmexec"
+)
 
 type noCallHelmExec struct {
 }
@@ -113,3 +117,8 @@ func (helm *noCallHelmExec) IsVersionAtLeast(versionStr string) bool {
 	helm.doPanic()
 	return false
 }
+
+func (helm *noCallHelmExec) ShowChart(chartPath string) (chart.Metadata, error) {
+	helm.doPanic()
+	return chart.Metadata{}, nil
+}
