@@ -12,8 +12,6 @@ type TestOptions struct {
 	Concurrency int
 	// SkipDeps is the skip deps flag
 	SkipDeps bool
-	// Args is the args to pass to helm lint
-	Args string
 	// Cleanup is the cleanup flag
 	Cleanup bool
 	// Logs is the logs flagj
@@ -52,11 +50,6 @@ func (t *TestImpl) SkipDeps() bool {
 	return t.TestOptions.SkipDeps
 }
 
-// Args returns the args
-func (t *TestImpl) Args() string {
-	return t.TestOptions.Args
-}
-
 // Cleanup returns the cleanup
 func (t *TestImpl) Cleanup() bool {
 	return t.TestOptions.Cleanup
