@@ -2,6 +2,7 @@ package state
 
 import (
 	"fmt"
+	"io"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -18,7 +19,7 @@ import (
 	"github.com/helmfile/helmfile/pkg/testhelper"
 )
 
-var logger = helmexec.NewLogger(os.Stdout, "warn")
+var logger = helmexec.NewLogger(io.Discard, "warn")
 var valsRuntime, _ = vals.New(vals.Options{CacheSize: 32})
 
 func injectFs(st *HelmState, fs *testhelper.TestFs) *HelmState {
@@ -1914,7 +1915,7 @@ generated: 2019-05-16T15:42:45.50486+09:00
 		return generatedDir, nil
 	}
 
-	logger := helmexec.NewLogger(os.Stderr, "debug")
+	logger := helmexec.NewLogger(io.Discard, "debug")
 	basePath := "/src"
 	state := &HelmState{
 		basePath: basePath,
@@ -1980,7 +1981,7 @@ generated: 2019-05-16T15:42:45.50486+09:00
 }
 
 func TestHelmState_ResolveDeps_NoLockFile(t *testing.T) {
-	logger := helmexec.NewLogger(os.Stderr, "debug")
+	logger := helmexec.NewLogger(io.Discard, "debug")
 	state := &HelmState{
 		basePath: "/src",
 		FilePath: "/src/helmfile.yaml",
@@ -2029,6 +2030,56 @@ func TestHelmState_ResolveDeps_NoLockFile(t *testing.T) {
 	}
 }
 
+func TestHelmState_ResolveDeps_NoLockFile_WithCustomLockFile(t *testing.T) {
+	logger := helmexec.NewLogger(io.Discard, "debug")
+	state := &HelmState{
+		basePath: "/src",
+		FilePath: "/src/helmfile.yaml",
+		ReleaseSetSpec: ReleaseSetSpec{
+			LockFile: "custom-lock-file",
+			Releases: []ReleaseSpec{
+				{
+					Chart: "./..",
+				},
+				{
+					Chart: "../examples",
+				},
+				{
+					Chart: "../../helmfile",
+				},
+				{
+					Chart: "published",
+				},
+				{
+					Chart: "published/deeper",
+				},
+				{
+					Chart: "stable/envoy",
+				},
+			},
+			Repositories: []RepositorySpec{
+				{
+					Name: "stable",
+					URL:  "https://kubernetes-charts.storage.googleapis.com",
+				},
+			},
+		},
+		logger: logger,
+		fs: &filesystem.FileSystem{
+			ReadFile: func(f string) ([]byte, error) {
+				if f != "custom-lock-file" {
+					return nil, fmt.Errorf("stub: unexpected file: %s", f)
+				}
+				return nil, os.ErrNotExist
+			},
+		},
+	}
+
+	_, err := state.ResolveDeps()
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+}
 func TestHelmState_ReleaseStatuses(t *testing.T) {
 	tests := []struct {
 		name     string
@@ -2691,3 +2742,269 @@ func TestGenerateOutputFilePath(t *testing.T) {
 		})
 	}
 }
+
+func TestFullFilePath(t *testing.T) {
+	fs := testhelper.NewTestFs(map[string]string{})
+	tests := []struct {
+		basePath string
+		filePath string
+		fs       *filesystem.FileSystem
+		expected string
+	}{
+		{
+			basePath: ".",
+			filePath: "helmfile.yaml",
+			expected: "helmfile.yaml",
+		},
+		{
+			basePath: "./test-1/",
+			filePath: "helmfile.yaml",
+			expected: "test-1/helmfile.yaml",
+		},
+		{
+			basePath: "/test-2/",
+			filePath: "helmfile.yaml",
+			expected: "/test-2/helmfile.yaml",
+		},
+		{
+			basePath: "./test-3/",
+			filePath: "helmfile.yaml",
+			fs:       fs.ToFileSystem(),
+			expected: "/path/to/test-3/helmfile.yaml",
+		},
+		{
+			basePath: "/test-4/",
+			filePath: "helmfile.yaml",
+			fs:       fs.ToFileSystem(),
+			expected: "/path/to/test-4/helmfile.yaml",
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.expected, func(t *testing.T) {
+			st := &HelmState{
+				basePath: tt.basePath,
+				FilePath: tt.filePath,
+				fs:       tt.fs,
+			}
+			actual, err := st.FullFilePath()
+			require.Equalf(t, actual, tt.expected, "FullFilePath() got = %v, want %v", actual, tt.expected)
+			require.Equalf(t, err, nil, "error %v", err)
+		})
+	}
+}
+
+func TestGetOCIQualifiedChartName(t *testing.T) {
+	tests := []struct {
+		state    HelmState
+		expected []struct {
+			qualifiedChartName string
+			chartName          string
+			chartVersion       string
+		}
+	}{
+		{
+			state: HelmState{
+				ReleaseSetSpec: ReleaseSetSpec{
+					Repositories: []RepositorySpec{},
+					Releases: []ReleaseSpec{
+						{
+							Chart:   "oci://registry/chart-path/chart-name",
+							Version: "0.1.2",
+						},
+					},
+				},
+			},
+			expected: []struct {
+				qualifiedChartName string
+				chartName          string
+				chartVersion       string
+			}{
+				{"registry/chart-path/chart-name:0.1.2", "chart-name", "0.1.2"},
+			},
+		},
+		{
+			state: HelmState{
+				ReleaseSetSpec: ReleaseSetSpec{
+					Repositories: []RepositorySpec{
+						{
+							Name: "oci-repo",
+							URL:  "registry/chart-path",
+							OCI:  true,
+						},
+					},
+					Releases: []ReleaseSpec{
+						{
+							Chart:   "oci-repo/chart-name",
+							Version: "0.1.2",
+						},
+					},
+				},
+			},
+			expected: []struct {
+				qualifiedChartName string
+				chartName          string
+				chartVersion       string
+			}{
+				{"registry/chart-path/chart-name:0.1.2", "chart-name", "0.1.2"},
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(fmt.Sprintf("%+v", tt.expected), func(t *testing.T) {
+			for i, r := range tt.state.Releases {
+				qualifiedChartName, chartName, chartVersion := tt.state.getOCIQualifiedChartName(&r)
+				require.Equalf(t, qualifiedChartName, tt.expected[i].qualifiedChartName, "qualifiedChartName got = %v, want %v", qualifiedChartName, tt.expected[i].qualifiedChartName)
+				require.Equalf(t, chartName, tt.expected[i].chartName, "chartName got = %v, want %v", chartName, tt.expected[i].chartName)
+				require.Equalf(t, chartVersion, tt.expected[i].chartVersion, "chartVersion got = %v, want %v", chartVersion, tt.expected[i].chartVersion)
+			}
+		})
+	}
+}
+
+func TestGenerateChartPath(t *testing.T) {
+	tests := []struct {
+		testName          string
+		chartName         string
+		release           *ReleaseSpec
+		outputDir         string
+		outputDirTemplate string
+		wantErr           bool
+		expected          string
+	}{
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndDefaultReleaseVersion",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseVersion",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", Version: "0.0.0"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/release-name/chart-name/0.0.0",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseTillerNamespace",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", TillerNamespace: "tiller-namespace"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/tiller-namespace/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseNamespace",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", Namespace: "release-namespace"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/release-namespace/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseTillerNamespaceAndGivenReleaseNamespace",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", TillerNamespace: "tiller-namespace", Namespace: "release-namespace"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/tiller-namespace/release-namespace/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseKubeContext",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", KubeContext: "kube-context"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/kube-context/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseNamespaceAndGivenReleaseKubeContext",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", Namespace: "release-namespace", KubeContext: "kube-context"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/release-namespace/kube-context/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseTillerNamespaceAndGivenReleaseKubeContext",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", TillerNamespace: "tiller-namespace", KubeContext: "kube-context"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/tiller-namespace/kube-context/release-name/chart-name/latest",
+		},
+		{
+			testName:  "PathGeneratedWithGivenOutputDirAndGivenReleaseTillerNamespaceAndGivenReleaseNamespaceAndGivenReleaseKubeContext",
+			chartName: "chart-name",
+			release:   &ReleaseSpec{Name: "release-name", TillerNamespace: "tiller-namespace", Namespace: "release-namespace", KubeContext: "kube-context"},
+			outputDir: "/output-dir",
+			wantErr:   false,
+			expected:  "/output-dir/tiller-namespace/release-namespace/kube-context/release-name/chart-name/latest",
+		},
+		{
+			testName:          "PathGeneratedWithGivenOutputDirAndGivenOutputDirTemplateWithFieldNameOutputDir",
+			chartName:         "chart-name",
+			release:           &ReleaseSpec{Name: "release-name"},
+			outputDir:         "/output-dir",
+			outputDirTemplate: "{{ .OutputDir }}",
+			wantErr:           false,
+			expected:          "/output-dir",
+		},
+		{
+			testName:          "PathGeneratedWithGivenOutputDirAndGivenOutputDirTemplateWithFieldNamesOutputDirAndReleaseName",
+			chartName:         "chart-name",
+			release:           &ReleaseSpec{Name: "release-name"},
+			outputDir:         "/output-dir",
+			outputDirTemplate: "{{ .OutputDir }}/{{ .Release.Name }}",
+			wantErr:           false,
+			expected:          "/output-dir/release-name",
+		},
+		{
+			testName:          "PathGeneratedWithGivenOutputDirTemplateWithFieldNamesOutputDir",
+			chartName:         "chart-name",
+			release:           &ReleaseSpec{Name: "release-name"},
+			outputDirTemplate: "{{ .OutputDir }}",
+			wantErr:           false,
+			expected:          "",
+		},
+		{
+			testName:          "PathGeneratedWithGivenOutputDirTemplateWithFieldNameReleaseName",
+			chartName:         "chart-name",
+			release:           &ReleaseSpec{Name: "release-name"},
+			outputDirTemplate: "{{ .Release.Name }}",
+			wantErr:           false,
+			expected:          "release-name",
+		},
+		{
+			testName:          "PathGeneratedWithGivenOutputDirTemplateWithStringAndFieldNameReleaseName",
+			chartName:         "chart-name",
+			release:           &ReleaseSpec{Name: "release-name"},
+			outputDirTemplate: "./charts/{{ .Release.Name }}",
+			wantErr:           false,
+			expected:          "./charts/release-name",
+		},
+		{
+			testName:          "ErrorReturnedWithGivenInvalidOutputDirTemplate",
+			chartName:         "chart-name",
+			release:           &ReleaseSpec{Name: "release-name"},
+			outputDirTemplate: "{{ .OutputDir }",
+			wantErr:           true,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.testName, func(t *testing.T) {
+			got, err := generateChartPath(tt.chartName, tt.outputDir, tt.release, tt.outputDirTemplate)
+
+			if tt.wantErr {
+				require.Errorf(t, err, "GenerateChartPath() error \"%v\", want error", err)
+			} else {
+				require.NoError(t, err, "GenerateChartPath() error \"%v\", want no error", err)
+			}
+			require.Equalf(t, tt.expected, got, "GenerateChartPath() got \"%v\", want \"%v\"", got, tt.expected)
+		})
+	}
+}
